@@ -15,26 +15,35 @@ nsga2 <- function(fn, idim, odim, ...,
                   cprob=0.7, cdist=5,
                   mprob=0.2, mdist=10,
                   vectorized=FALSE) {
-  ff <- function(x) {
-    if (vectorized) {
-      fn(x, ...)
-    } else {
-      apply(x, 1, fn, ...)
-    }
+  ff <- if (vectorized)  {
+    function(x) fn(x, ...)
+  } else {
+    function(x) apply(x, 1, fn, ...)
   }
-  
-  cf <- function(x) {
-    if (vectorized) {
-      constraints(x, ...)
-    } else {
-      apply(x, 1, constraints, ...)
-    }
+
+  cf <- if (vectorized) {
+    function(x) constraints(x, ...)
+  } else {
+    function(x) apply(x, 1, constraints, ...)
   }
 
   ## Make sure popsize is a multiple of 4
   if (popsize %% 4 != 0)
     stop("Population size must be a multiple of 4")
 
+  ## Check bounding box
+  if (any(is.infinite(lower.bounds)) || any(is.infinite(upper.bounds))) {
+    warning("While it is possible to optimize an unconstrained problem, it is not recommended. Please consider adding finite upper and lower bounds.")
+    sane_lower <- -.Machine$double.xmax / 4
+    sane_upper <- .Machine$double.xmax / 4
+    lower.bounds <- ifelse(lower.bounds == -Inf, sane_lower,
+                           ifelse(lower.bounds == Inf, sane_upper,
+                                  lower.bounds))
+
+    upper.bounds <- ifelse(upper.bounds == -Inf, sane_lower,
+                           ifelse(upper.bounds == Inf, sane_upper,
+                                  upper.bounds))
+  }
   ## Lag generations:
   ## C source expects each element of generations to be the number of
   ## generations to go forward before saving the next result set. This is
@@ -44,10 +53,10 @@ nsga2 <- function(fn, idim, odim, ...,
 
   if (!all(generations > 0))
     stop("Cannot go back in time! Your generations argument must be sorted!")
-  
+
   ## Set cdim = 0 if no cfn was given:
   if (is.null(constraints)) cdim <- 0
-  
+
   res <- .Call(do_nsga2,
                ff, cf, sys.frame(),
                as.integer(odim),
@@ -71,7 +80,7 @@ nsga2 <- function(fn, idim, odim, ...,
   return (res)
 }
 
-plot.nsga2 <- function(x, ...) {  
+plot.nsga2 <- function(x, ...) {
   v <- x$value
   o <- x$pareto.optimal
   d <- ncol(v)
